@@ -6,11 +6,11 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.12
+Version 1.18
 
 =cut
 
-our $VERSION = "1.12";
+our $VERSION = "1.18";
 
 =head1 SYNOPSIS
 
@@ -75,6 +75,9 @@ you can also use any of L<LWP::UserAgent>'s methods.
 
     $mech->add_header($name => $value);
 
+Please note that Mech does NOT support JavaScript.  Please check the
+FAQ in WWW::Mechanize::FAQ for more.
+
 =head1 IMPORTANT LINKS
 
 =over 4
@@ -100,6 +103,7 @@ use LWP::UserAgent 2.003;
 use HTML::Form 1.00;
 use HTML::TokeParser;
 use URI::URL;
+use UNIVERSAL qw( isa );
 
 use base 'LWP::UserAgent';
 
@@ -198,7 +202,8 @@ sub new {
     while ( my($key,$value) = each %passed_parms ) {
         if ( exists $mech_parms{$key} ) {
             $mech_parms{$key} = $value;
-        } else {
+        }
+        else {
             $parent_parms{$key} = $value;
         }
     }
@@ -271,7 +276,8 @@ sub agent_alias {
 
     if ( defined $known_agents{$alias} ) {
         return $self->agent( $known_agents{$alias} );
-    } else {
+    }
+    else {
         $self->warn( qq{Unknown agent alias "$alias"} );
         return $self->agent();
     }
@@ -493,13 +499,16 @@ sub content {
                 $tree->eof();
                 $tree->elementify(); # just for safety
                 $content = $tree->as_text();
-            } else {
+            }
+            else {
                 $self->die( qq{Unknown format parameter "$arg"} );
             };
-        } elsif ($cmd eq 'base_href') {
+        }
+        elsif ($cmd eq 'base_href') {
             $arg ||= $self->base;
             $content=~s/<head>/<head>\n<base href="$arg">/;
-        } else {
+        }
+        else {
             $self->die( qq{Unknown named argument "$cmd"} );
         }
     }
@@ -519,6 +528,9 @@ links.  In scalar context, returns an array reference of all links.
 
 sub links {
     my $self = shift ;
+
+    $self->_extract_links() unless $self->{_extracted_links};
+
     return @{$self->{links}} if wantarray;
     return $self->{links};
 }
@@ -688,7 +700,8 @@ sub find_link {
         if ( _match_any_link_parms($link,\%parms) ) {
             if ( $wantall ) {
                 push( @matches, $link );
-            } else {
+            }
+            else {
                 ++$nmatches;
                 return $link if $nmatches >= $parms{n};
             }
@@ -751,7 +764,8 @@ sub _clean_keys {
                 delete $parms->{$key};
                 next;
             }
-        } else {
+        }
+        else {
             if ( $val_regex ) {
                 $self->warn( qq{$val passed as '$key' is a regex} );
                 delete $parms->{$key};
@@ -799,6 +813,9 @@ images.  In scalar context, returns an array reference of all images.
 
 sub images {
     my $self = shift ;
+
+    $self->_extract_images() unless $self->{_extracted_images};
+
     return @{$self->{images}} if wantarray;
     return $self->{images};
 }
@@ -882,7 +899,8 @@ sub find_image {
         if ( _match_any_image_parms($image,\%parms) ) {
             if ( $wantall ) {
                 push( @matches, $image );
-            } else {
+            }
+            else {
                 ++$nmatches;
                 return $image if $nmatches >= $parms{n};
             }
@@ -970,9 +988,10 @@ sub form_number {
     if ($self->{forms}->[$form-1]) {
         $self->{form} = $self->{forms}->[$form-1];
         return $self->{form};
-    } else {
+    }
+    else {
         $self->warn( "There is no form numbered $form" );
-        return undef;
+        return;
     }
 }
 
@@ -996,7 +1015,8 @@ sub form_name {
         $self->warn( "There are ", scalar @matches, " forms named $form.  The first one was used." )
             if @matches > 1;
         return $self->{form} = $matches[0];
-    } else {
+    }
+    else {
         $self->warn( qq{ There is no form named "$form"} );
         return undef;
     }
@@ -1023,10 +1043,12 @@ sub field {
     my $form = $self->{form};
     if ($number > 1) {
         $form->find_input($name, undef, $number)->value($value);
-    } else {
+    }
+    else {
         if ( ref($value) eq "ARRAY" ) {
             $form->param($name, $value);
-        } else {
+        }
+        else {
             $form->value($name => $value);
         }
     }
@@ -1081,7 +1103,8 @@ sub select {
             # (see INPUTS section of `perldoc HTML::Form`)
             if (@inputs == 1) {
                 @values = $inputs[0]->possible_values();
-            } else {
+            }
+            else {
                 foreach my $input (@inputs) {
                     my @possible = $input->possible_values();
                     push @values, pop @possible;
@@ -1098,13 +1121,16 @@ sub select {
                     }
                     push @$value, $values[$_ - 1];  # might be undef
                 }
-            } elsif (!ref($n) && $n =~ /^\d+$/) {
+            }
+            elsif (!ref($n) && $n =~ /^\d+$/) {
                 $value = $values[$n - 1];           # might be undef
-            } else {
+            }
+            else {
                 $self->warn('"n" value is not a positive integer or an array ref');
                 return;
             }
-        } else {
+        }
+        else {
             $self->warn('Hash value is invalid');
             return;
         }
@@ -1147,7 +1173,8 @@ sub set_fields {
         if ( ref $value eq 'ARRAY' ) {
             $form->find_input( $field, undef,
                          $value->[1])->value($value->[0] );
-        } else {
+        }
+        else {
             $form->value($field => $value);
         }
     } # while
@@ -1208,7 +1235,8 @@ sub set_visible {
                     last;
                 }
             } # while
-        } else {
+        }
+        else {
             while ( my $input = shift @inputs ) {
                 next if $input->type eq 'hidden';
                 $input->value( $value );
@@ -1291,7 +1319,8 @@ sub value {
     my $form = $self->{form};
     if ( $number > 1 ) {
         return $form->find_input( $name, undef, $number )->value();
-    } else {
+    }
+    else {
         return $form->value( $name );
     }
 } # value
@@ -1372,12 +1401,15 @@ sub click_button {
     my $request;
     if ( $args{name} ) {
         $request = $form->click( $args{name}, $args{x}, $args{y} );
-    } elsif ( $args{number} ) {
+    }
+    elsif ( $args{number} ) {
         my $input = $form->find_input( undef, 'submit', $args{number} );
         $request = $input->click( $form, $args{x}, $args{y} );
-    } elsif ( $args{input} ) {
+    }
+    elsif ( $args{input} ) {
         $request = $args{input}->click( $form, $args{x}, $args{y} );
-    } elsif ( $args{value} ) {
+    }
+    elsif ( $args{value} ) {
         my $i = 1;
         while ( my $input = $form->find_input(undef, 'submit', $i) ) {
             if ( $args{value} && ($args{value} eq $input->value) ) {
@@ -1458,14 +1490,14 @@ sub submit_form {
     }
 
     if ( my $form_number = $args{'form_number'} ) {
-        $self->form_number( $form_number ) ;
+        $self->form_number( $form_number ) or die;
     }
     elsif ( my $form_name = $args{'form_name'} ) {
-        $self->form_name( $form_name ) ;
+        $self->form_name( $form_name ) or die;
     }
 
     if ( my $fields = $args{'fields'} ) {
-        if ( ref $fields eq 'HASH' ) {
+        if ( isa( $fields, 'HASH' ) ) {
             $self->set_fields( %{$fields} ) ;
         } # TODO: What if it's not a hash?  We just ignore it silently?
     }
@@ -1473,7 +1505,8 @@ sub submit_form {
     my $response;
     if ( $args{button} ) {
         $response = $self->click( $args{button}, $args{x} || 0, $args{y} || 0 );
-    } else {
+    }
+    else {
         $response = $self->submit();
     }
 
@@ -1696,8 +1729,8 @@ sub update_html {
         }
     }
     $self->{form}  = $self->{forms}->[0];
-    $self->_extract_links();
-    $self->_extract_images();
+    $self->{_extracted_links} = 0;
+    $self->{_extracted_images} = 0;
 
     return;
 }
@@ -1728,11 +1761,13 @@ sub follow {
     if ( $link =~ /^\d+$/ ) { # is a number?
         if ($link <= $#links) {
             $thislink = $links[$link];
-        } else {
+        }
+        else {
             $self->warn( "Link number $link is greater than maximum link $#links on this page ($self->{uri})" );
             return;
         }
-    } else {                        # user provided a regexp
+    }
+    else {                        # user provided a regexp
         LINK: foreach my $l (@links) {
             if ( defined($l->[1]) && $l->[1] =~ /$link/) {
                 $thislink = $l;     # grab first match
@@ -1799,7 +1834,8 @@ sub _update_page {
     if ( $res->is_success ) {
         $self->{uri} = $self->{redirected_uri};
         $self->{last_uri} = $self->{uri};
-    } else {
+    }
+    else {
         if ( $self->{autocheck} ) {
             $self->die( "Error ", $request->method, "ing ", $request->uri, ": ", $res->message );
         }
@@ -1808,7 +1844,8 @@ sub _update_page {
     $self->_reset_page;
     if ($self->is_html) {
         $self->update_html($res->content);
-    } else {
+    }
+    else {
         $self->{content} = $res->content;
     }
 
@@ -1841,7 +1878,8 @@ sub _modify_request {
     while ( my($key,$value) = each %{$self->{headers}} ) {
         if ( defined $value ) {
             $req->header( $key => $value );
-        } else {
+        }
+        else {
             $req->remove_header( $key );
         }
     }
@@ -1871,7 +1909,10 @@ Resets the internal fields that track page parsed stuff.
 sub _reset_page {
     my $self = shift;
 
+    $self->{_extracted_links} = 0;
+    $self->{_extracted_images} = 0;
     $self->{links} = [];
+    $self->{images} = [];
     $self->{forms} = [];
     delete $self->{form};
 
@@ -1896,14 +1937,17 @@ my %link_tags = (
 sub _extract_links {
     my $self = shift;
 
-    my $parser = HTML::TokeParser->new(\$self->{content});
 
     $self->{links} = [];
+    if ( defined $self->{content} ) {
+        my $parser = HTML::TokeParser->new(\$self->{content});
+        while ( my $token = $parser->get_tag( keys %link_tags ) ) {
+            my $link = $self->_link_from_token( $token, $parser );
+            push( @{$self->{links}}, $link ) if $link;
+        } # while
+    }
 
-    while ( my $token = $parser->get_tag( keys %link_tags ) ) {
-        my $link = $self->_link_from_token( $token, $parser );
-        push( @{$self->{links}}, $link ) if $link;
-    } # while
+    $self->{_extracted_links} = 1;
 
     return;
 }
@@ -1917,14 +1961,17 @@ my %image_tags = (
 sub _extract_images {
     my $self = shift;
 
-    my $parser = HTML::TokeParser->new(\$self->{content});
-
     $self->{images} = [];
 
-    while ( my $token = $parser->get_tag( keys %image_tags ) ) {
-        my $image = $self->_image_from_token( $token, $parser );
-        push( @{$self->{images}}, $image ) if $image;
-    } # while
+    if ( defined $self->{content} ) {
+        my $parser = HTML::TokeParser->new(\$self->{content});
+        while ( my $token = $parser->get_tag( keys %image_tags ) ) {
+            my $image = $self->_image_from_token( $token, $parser );
+            push( @{$self->{images}}, $image ) if $image;
+        } # while
+    }
+
+    $self->{_extracted_images} = 1;
 
     return;
 }
@@ -1989,7 +2036,8 @@ sub _link_from_token {
 
         if ( $content =~ /^\d+\s*;\s*url\s*=\s*(\S+)/i ) {
             $url = $1;
-        } else {
+        }
+        else {
             undef $url;
         }
     } # meta
@@ -2234,6 +2282,8 @@ Just like Mech, but using Microsoft Internet Explorer to do the work.
 
 =item * L<WWW::Bugzilla>
 
+=item * L<WWW::CheckSite>
+
 =item * L<WWW::Google::Groups>
 
 =item * L<WWW::Hotmail>
@@ -2299,11 +2349,13 @@ Leland Johnson,
 Joshua Gatcomb,
 Julien Beasley,
 Abe Timmerman,
+Peter Stevens,
+Pete Krawczyk,
 and the late great Iain Truskett.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005 Andy Lester. All rights reserved. This program is
+Copyright (c) 2006 Andy Lester. All rights reserved. This program is
 free software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
